' ------------------------------------------------------------------------------------
' irr::core::quaternion wrappers (done)
' ------------------------------------------------------------------------------------

Public

Type Quaternion Extends IrrBase Final
	
	'--------------------------------------------
	' operator overloads
	'--------------------------------------------

	Method EqEq:Int(pOther:Quaternion)
		Return IrrCore_Quaternion_EqEq(handle,pOther.handle)
	EndMethod
	Method EqQuat(pOther:Quaternion)
		IrrCore_Quaternion_EqQuat(handle,pOther.handle)
	EndMethod
	Method EqMat(pOtherMat:Matrix4)
		IrrCore_Quaternion_EqMat(handle,pOtherMat.handle)
	EndMethod
	Method Plus:Quaternion(pOther:Quaternion)
		Return createFromHandle(IrrCore_Quaternion_Plus(handle,pOther.handle))
	EndMethod
	Method MultQuat:Quaternion(pOther:Quaternion)
		Return createFromHandle(IrrCore_Quaternion_MultQuat(handle,pOther.handle))
	EndMethod
	Method MultFloat:Quaternion(s:Float)
		Return createFromHandle(IrrCore_Quaternion_MultFloat(handle,s))
	EndMethod
	Method MultVect:Vector3df(pVect:Vector3df)
		Return Vector3df.createFromHandle(IrrCore_Quaternion_MultVect(handle,pVect.handle))
	EndMethod
	Method MultEqFloat(s:Float)
		IrrCore_Quaternion_MultEqFloat(handle,s)
	EndMethod
	Method MultEqQuat(pOther:Quaternion)
		IrrCore_Quaternion_MultEqQuat(handle,pOther.handle)
	EndMethod

	Method Equals:Int(other:Quaternion, tolerance:Float = 0.000001)
		Return IrrCore_Quaternion_equals(handle, _HANDLE(other), tolerance)
	EndMethod

	Method NotEq:Int(other:Quaternion)
		Return IrrCore_Quaternion_NotEq(handle, _HANDLE(other))
	EndMethod

	'--------------------------------------------
	' get/manipulation routines
	'--------------------------------------------

	Method dotProduct:Float(pOther:Quaternion)	
		Return IrrCore_Quaternion_dotProduct(handle,pOther.handle)
	EndMethod
	Method setFrom4Vals:Quaternion(x:Float,y:Float,z:Float,w:Float)
		IrrCore_Quaternion_setFrom4Vals(handle,x,y,z,w)
		Return Self
	EndMethod
	Method setFrom3Vals:Quaternion(x:Float,y:Float,z:Float)
		IrrCore_Quaternion_setFrom3Vals(handle,x,y,z)
		Return Self
	EndMethod
	Method setFromVect:Quaternion(vec:Vector3df)
		IrrCore_Quaternion_setFromVect(handle, vec.handle)
		Return Self
	EndMethod

	Method setFromOther:Quaternion(other:Quaternion)
		IrrCore_Quaternion_setFromOther(handle, _HANDLE(other)) 
		Return Self
	EndMethod

	Method normalize:Quaternion()
		Return createFromHandle(IrrCore_Quaternion_normalize(handle),False)
	EndMethod
	Method getMatrix:Matrix4()
		Return Matrix4.createFromHandle(IrrCore_Quaternion_getMatrix(handle))
	EndMethod
	Method getMatrixToDest(dest:Matrix4, translation:Vector3df)
		IrrCore_Quaternion_getMatrixToDest(handle, dest.handle, translation.handle)
	EndMethod
	Method getMatrix_transposed(dest:Matrix4)
		IrrCore_Quaternion_getMatrix_transposed(handle,dest.handle)
	EndMethod
	Method makeInverse:Quaternion()
		IrrCore_Quaternion_makeInverse(handle)
		Return Self
	EndMethod	
	Method slerp:Quaternion(pQ1:Quaternion,pQ2:Quaternion,interpolate:Float)
		IrrCore_Quaternion_slerp(handle,pQ1.handle,pQ2.handle,interpolate)
		Return Self
	EndMethod
	Method fromAngleAxis:Quaternion(angle:Float,pAxis:Vector3df)
		IrrCore_Quaternion_fromAngleAxis(handle,angle,pAxis.handle)
		Return Self
	EndMethod
	Method toEuler(pEuler:Vector3df)
		IrrCore_Quaternion_toEuler(handle,pEuler.handle)
	EndMethod
	Method makeIdentity:Quaternion()
		IrrCore_Quaternion_makeIdentity(handle)
		Return Self
	EndMethod
	Method getX:Float()
		Return IrrCore_Quaternion_getX:Float(handle)
	EndMethod
	Method getY:Float()
		Return IrrCore_Quaternion_getY:Float(handle)
	EndMethod
	Method getZ:Float()
		Return IrrCore_Quaternion_getZ:Float(handle)
	EndMethod
	Method getW:Float()
		Return IrrCore_Quaternion_getW:Float(handle)
	EndMethod

	Method toAngleAxis(angle:Float Var,axis:Vector3df)
		IrrCore_Quaternion_toAngleAxis(handle,Varptr(angle),axis.handle)
	EndMethod

	Method rotationFromTo:Quaternion(fromV:Vector3df, toV:Vector3df)
		IrrCore_Quaternion_rotationFromTo(handle, fromV.handle, toV.handle)
		Return Self
	EndMethod

	Method getMatrixCenter(dest:Matrix4, center:Vector3df, translation:Vector3df)
		IrrCore_Quaternion_getMatrixCenter(handle, _HANDLE(dest), _HANDLE(center), _HANDLE(translation))
	EndMethod

	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------

	Function create:Quaternion()
		Return createFromHandle(IrrCore_Quaternion_new())
	EndFunction
	Function createFrom3Vals:Quaternion(x:Float,y:Float,z:Float)
		Return createFromHandle(IrrCore_Quaternion_newFrom3Vals(x,y,z))
	EndFunction
	Function createFrom4Vals:Quaternion(x:Float,y:Float,z:Float,w:Float)
		Return createFromHandle(IrrCore_Quaternion_newFrom4Vals(x,y,z,w))
	EndFunction
	Function createFromMatrix:Quaternion(pMat:Matrix4)
		Return createFromHandle(IrrCore_Quaternion_newFromMatrix(pMat.handle))
	EndFunction
	Function createFromVect:Quaternion(vect:Vector3df)
		Return createFromHandle(IrrCore_Quaternion_newFromVect(vect.handle),True)
	EndFunction

	Function createFromHandle:Quaternion(pQuat:Int,bMustDel:Int=True)
		If Not pQuat Then Return Null
		Local oRetVal:Quaternion=New Quaternion
		oRetVal.attach(pQuat,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_Quaternion_del(handle)
	EndMethod	

EndType