' ------------------------------------------------------------------------------------
' irr::core::triangle3di wrappers (done)
' ------------------------------------------------------------------------------------
Public

Type Triangle3di Extends IrrBase Final
	
	Method closestPointOnTriangle:Vector3di(pVect:Vector3di)
		Return Vector3di.createFromHandle(IrrCore_Triangle3di_closestPointOnTriangle(handle,pVect.handle))
	EndMethod

	Method isTotalInsideBox:Int(pBox:AABbox3di)
		Return IrrCore_Triangle3di_isTotalInsideBox(handle,pBox.handle)
	EndMethod

	Method isTotalOutsideBox:Int(pBox:AABbox3di)
		Return IrrCore_Triangle3di_isTotalOutsideBox(handle,pBox.handle)
	EndMethod
		
	Method isFrontFacing:Int(pLookDirection:Vector3di)
		Return IrrCore_Triangle3di_isFrontFacing(handle,pLookDirection.handle)
	EndMethod
	
	Method isPointInside:Int(pVect:Vector3di)
		Return IrrCore_Triangle3di_isPointInside(handle,pVect.handle)
	EndMethod

	Method isPointInsideFast:Int(pVect:Vector3di)
		Return IrrCore_Triangle3di_isPointInsideFast(handle,pVect.handle)
	EndMethod
	
	Method EqEq:Int(pTriOther:Triangle3di)
		Return IrrCore_Triangle3di_EqEq(handle,pTriOther.handle)
	EndMethod

	Method NotEq:Int(pTriOther:Triangle3di)
		Return IrrCore_Triangle3di_NotEq(handle,pTriOther.handle)
	EndMethod

	'--------------------------------------------
	' get routines
	'--------------------------------------------

	Method getIntersectionWithLimitedLine:Int(pLine:Line3di,pOutIntersection:Vector3di)
		Return IrrCore_Triangle3di_getIntersectionWithLimitedLine(handle,pLine.handle,pOutIntersection.handle)
	EndMethod

	Method getIntersectionWithLine:Int(pLinePoint:Vector3di,pLineVect:Vector3di,pOutIntersection:Vector3di)
		Return IrrCore_Triangle3di_getIntersectionWithLine(handle,pLinePoint.handle,pLineVect.handle,pOutIntersection.handle)
	EndMethod

	Method getIntersectionOfPlaneWithLine:Int(pLinePoint:Vector3di,pLineVect:Vector3di,pOutIntersection:Vector3di)
		Return IrrCore_Triangle3di_getIntersectionOfPlaneWithLine(handle,pLinePoint.handle,pLineVect.handle,pOutIntersection.handle)
	EndMethod

	Method getNormal:Vector3di()
		Return Vector3di.createFromHandle(IrrCore_Triangle3di_getNormal(handle))
	EndMethod
	
	Method getPlane:Plane3di()
		Return Plane3di.createFromHandle(IrrCore_Triangle3di_getPlane(handle))
	EndMethod
	
	Method getPointA:Vector3di()
		' set to not delete
		Return Vector3di.createFromHandle(IrrCore_Triangle3di_getPointA(handle),False)
	EndMethod
	
	Method getPointB:Vector3di()
		' set to not delete
		Return Vector3di.createFromHandle(IrrCore_Triangle3di_getPointB(handle),False)
	EndMethod

	Method getPointC:Vector3di()
		' set to not delete
		Return Vector3di.createFromHandle(IrrCore_Triangle3di_getPointC(handle),False)
	EndMethod

	Method getArea:Int()
		Return IrrCore_Triangle3di_getArea(handle)
	EndMethod

	'--------------------------------------------
	' set routines
	'--------------------------------------------

	Method setFromVect(pVect1:Vector3di,pVect2:Vector3di,pVect3:Vector3di)
		IrrCore_Triangle3di_setFromVect(handle,pVect1.handle,pVect2.handle,pVect3.handle)
	EndMethod
	
	Method setFromVals(x1:Int=0,y1:Int=0,z1:Int=0,x2:Int=0,y2:Int=0,z2:Int=0,x3:Int=0,y3:Int=0,z3:Int=0)
		IrrCore_Triangle3di_setFromVals(handle,x1,y1,z1,x2,y2,z2,x3,y3,z3)
	EndMethod

	Method setPointA(pVect:Vector3di)
		IrrCore_Triangle3di_setPointA(handle,pVect.handle)
	EndMethod

	Method setPointB(pVect:Vector3di)
		IrrCore_Triangle3di_setPointB(handle,pVect.handle)
	EndMethod

	Method setPointC(pVect:Vector3di)
		IrrCore_Triangle3di_setPointC(handle,pVect.handle)
	EndMethod

	'--------------------------------------------
	' create/delete routines
	'--------------------------------------------

	Function create:Triangle3di()
		Return createFromHandle(IrrCore_Triangle3di_new())
	EndFunction

	Function createFromVals:Triangle3di(x1:Int=0,y1:Int=0,z1:Int=0,x2:Int=0,y2:Int=0,z2:Int=0,x3:Int=0,y3:Int=0,z3:Int=0)
		Return createFromHandle(IrrCore_Triangle3di_newFromVals(x1,y1,z1,x2,y2,z2,x3,y3,z3))
	EndFunction
	
	Function createFromVect:Triangle3di(pVect1:Vector3di,pVect2:Vector3di,pVect3:Vector3di)
		Return createFromHandle(IrrCore_Triangle3di_newFromVect(pVect1.handle,pVect2.handle,pVect3.handle))
	EndFunction
	
	Function createFromHandle:Triangle3di(pTri:Int,bMustDel:Int=True)
		If Not pTri Then Return Null
		Local oRetVal:Triangle3di=New Triangle3di
		oRetVal.attach(pTri,bMustDel)
		Return oRetVal
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrCore_Triangle3di_del(handle)
	EndMethod	
EndType



