Public

Type ILightManager Extends IReferenceCounted
	
	' --------------------------------------------------------------------------------
	' override these in extended type to implement new functionality
	' --------------------------------------------------------------------------------
	
	Method OnPreRender(lightList:ILightSceneNode[]) Abstract

	Method OnPostRender() Abstract

	Method OnRenderPassPreRender(E_SCENE_NODE_RENDER_PASS:Int) Abstract
	
	Method OnRenderPassPostRender(E_SCENE_NODE_RENDER_PASS:Int) Abstract
	
	Method OnNodePreRender(node:ISceneNode) Abstract

	Method OnNodePostRender(node:ISceneNode) Abstract

	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrScene_BMXILightManager_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrScene_BMXILightManager_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrScene_BMXILightManager_grab(handle)
	EndMethod

	Method Delete()
		If (mustdelete) Then drop()
	EndMethod
	
	' --------------------------------------------------------------------------------
	' creation routines
	' --------------------------------------------------------------------------------

	' this should only be called on an extended type
	Function create:ILightManager(pGenFunc:Byte Ptr) Final
		Return createFromHandle(IrrScene_BMXILightManager_new(),True,pGenFunc)
	EndFunction

	Function createFromHandle:ILightManager(pHandle:Int,bMustDel:Int,pGenFunc:Byte Ptr=Null) Final
		If Not pHandle Then Return Null
		Local oRetVal:ILightManager=Null
		
		If pGenFunc
			' use the funtion pointer to generate
			Local generate:ILightManager()=pGenFunc
		
			oRetVal=generate()
		EndIf

		' turn binding on if this is the first and there was a gen func passed in
		oRetVal.bIsBound=bMustDel And pGenFunc
		
		oRetVal.attach(pHandle,bMustDel)
		
		Return oRetVal
	EndFunction

	' --------------------------------------------------------------------------------
	' bindings
	' --------------------------------------------------------------------------------

	Method bind(rec:Int) Final
		Super.bind(rec)
	EndMethod

	Method getBinderHandle:Int()
		Return IrrScene_BMXILightManager_getBinder(handle)
	EndMethod

	Function BoundOnPreRender(binding:Int, lightList:Int Ptr, lightCount:Int)
		' get the object whose node is firing
		Local tempobj:ILightManager = ILightManager(translate(binding))
		
		If tempobj<>Null
			' build an array of lights
			Local objs:ILightSceneNode[] = Null

			If lightCount > 0 Then		
				objs = New ILightSceneNode[lightCount]

				For Local i:Int=0 Until lightCount
					objs[i] = ILightSceneNode.createFromHandle(lightList[i], False)
				Next
			EndIf
							
  			' fire the event
	  		tempobj.OnPreRender(objs)
		Else
			Throw "Unable to find bound object"
		EndIf	
	EndFunction

	Function BoundOnPostRender(binding:Int) 
		' get the object whose node is firing
		Local tempobj:ILightManager = ILightManager(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		tempobj.OnPostRender()
		Else
			Throw "Unable to find bound object"
		EndIf	
	EndFunction

	Function BoundOnRenderPassPreRender(binding:Int, E_SCENE_NODE_RENDER_PASS:Int)
		' get the object whose node is firing
		Local tempobj:ILightManager = ILightManager(translate(binding))

		If tempobj<>Null
  			' fire the event
	  		tempobj.OnRenderPassPreRender(E_SCENE_NODE_RENDER_PASS)
		Else
			Throw "Unable to find bound object"
		EndIF		
	EndFunction
	
	Function BoundOnRenderPassPostRender(binding:Int, E_SCENE_NODE_RENDER_PASS:Int)
		' get the object whose node is firing
		Local tempobj:ILightManager = ILightManager(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		tempobj.OnRenderPassPostRender(E_SCENE_NODE_RENDER_PASS)
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction
	
	Function BoundOnNodePreRender(binding:Int, pISceneNode:Int)
		' get the object whose node is firing
		Local tempobj:ILightManager = ILightManager(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		tempobj.OnNodePreRender(ISceneNode.createFromHandle(pISceneNode, False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

	Function BoundOnNodePostRender(binding:Int, pISceneNode:Int)
		' get the object whose node is firing
		Local tempobj:ILightManager = ILightManager(translate(binding))
		
		If tempobj<>Null
  			' fire the event
	  		tempobj.OnNodePostRender(ISceneNode.createFromHandle(pISceneNode, False))
		Else
			Throw "Unable to find bound object"
		EndIf		
	EndFunction

EndType
