' ------------------------------------------------------------------------------------
'  irr::scene::IMeshManipulator wrappers
' ------------------------------------------------------------------------------------
Public

Type IMeshManipulator Extends IReferenceCounted Final

	Function createFromHandle:IMeshManipulator(pHandle:Int,bMustDel:Int=True)
		If Not pHandle Then Return Null
		Local oRetVal:IMeshManipulator=New IMeshManipulator
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction
	
	Method createMeshCopy:SMesh(mesh:IMesh)
		Return SMesh.createFromHandle(IrrScene_IMeshManipulator_createMeshCopy(handle,mesh.handle), True)
	EndMethod
	
	Method createMeshWithTangents:IMesh(mesh:IMesh, bRecalculateNormals:Int = False, bSmooth:Int = False, bAngleWeighted:Int = False)
		Return IMesh.createFromHandle(IrrScene_IMeshManipulator_createMeshWithTangents(handle,_HANDLE(mesh), bRecalculateNormals, bSmooth, bAngleWeighted), True)
	EndMethod

	Method createMeshUniquePrimitives:IMesh(mesh:IMesh)
		Return IMesh.createFromHandle(IrrScene_IMeshManipulator_createMeshUniquePrimitives(handle,mesh.handle), True)
	EndMethod

	Method flipSurfaces(mesh:IMesh)
		IrrScene_IMeshManipulator_flipSurfaces(handle,mesh.handle)
	EndMethod
	
	Method getPolyCountFromAnimated:Int(mesh:IAnimatedMesh)
		Return IrrScene_IMeshManipulator_getPolyCountFromAnimated(handle,mesh.handle)
	EndMethod
	
	Method getPolyCount:Int(mesh:IMesh)
		Return IrrScene_IMeshManipulator_getPolyCount(handle,mesh.handle)
	EndMethod
	
	Method makePlanarTextureMapping(mesh:IMesh,resolution:Float=0.001)
		IrrScene_IMeshManipulator_makePlanarTextureMapping(handle,mesh.handle,resolution)
	EndMethod
	
	Method recalculateNormals(mesh:IMesh,bSmooth:Int=False, bAngleWeighted:Int = False)
		IrrScene_IMeshManipulator_recalculateNormals(handle,mesh.handle,bSmooth, bAngleWeighted)
	EndMethod
	
	Method recalculateNormalsMeshBuffer(buffer:IMeshBuffer,bSmooth:Int=False, bAngleWeighted:Int = False)
		IrrScene_IMeshManipulator_recalculateNormalsMeshBuffer(handle,buffer.handle,bSmooth, bAngleWeighted)
	EndMethod

	Method scaleMesh(mesh:IMesh,scale:Vector3df)
		IrrScene_IMeshManipulator_scaleMesh(handle,mesh.handle,scale.handle)
	EndMethod
	
	Method setVertexColorAlpha(mesh:IMesh,alpha:Int)
		IrrScene_IMeshManipulator_setVertexColorAlpha(handle,mesh.handle,alpha)
	EndMethod

	Method setVertexColors(mesh:IMesh,color:SColor)
		IrrScene_IMeshManipulator_setVertexColors(handle,mesh.handle,color.handle)
	EndMethod

	Method transformMesh(mesh:IMesh,m:Matrix4)
		IrrScene_IMeshManipulator_transformMesh(handle,mesh.handle,m.handle)
	EndMethod
	
	Method createAnimatedMesh:IAnimatedMesh(mesh:IMesh,E_ANIMATED_MESH_TYPE:Int=EAMT_UNKNOWN)
		Return IAnimatedMesh.createFromHandle(IrrScene_IMeshManipulator_createAnimatedMesh(handle,mesh.handle,E_ANIMATED_MESH_TYPE),True)
	EndMethod

	Method createMeshWith2TCoords:IMesh(mesh:IMesh)
		Return IMesh.createFromHandle(IrrScene_IMeshManipulator_createMeshWith2TCoords(handle,mesh.handle), True)
	EndMethod
	
	Method createMeshWelded:IMesh(mesh:IMesh, tolerance:Float = 0.000001)
		Return IMesh.createFromHandle(IrrScene_IMeshManipulator_createMeshWelded(handle, mesh.handle, tolerance), True)
	EndMethod
	
	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrScene_IMeshManipulator_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrScene_IMeshManipulator_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrScene_IMeshManipulator_grab(handle)
	EndMethod
	
	Method Delete()
		If (mustdelete) Then drop()
	EndMethod
	
	Method scale(mesh:IMesh, factor:Vector3df)
		IrrScene_IMeshManipulator_scale(handle, mesh.handle, factor.handle)
	EndMethod
	
	Method scaleMeshBuffer(buffer:IMeshbuffer, factor:Vector3df)
		IrrScene_IMeshManipulator_scaleMeshBuffer(handle, buffer.handle, factor.handle)
	EndMethod
	
	Method scaleTCoords(mesh:IMesh, factor:Vector2df, level:Int = 1)
		IrrScene_IMeshManipulator_scaleTCoords(handle, mesh.handle, factor.handle, level)
	EndMethod
	
	Method scaleTCoordsMeshBuffer(buffer:IMeshBuffer, factor:Vector2df, level:Int = 1)
		IrrScene_IMeshManipulator_scaleTCoordsMeshBuffer(handle, buffer.handle, factor.handle, level)
	EndMethod
	
	Method transform(mesh:IMesh, matrix:Matrix4)
		IrrScene_IMeshManipulator_transform(handle, mesh.handle, matrix.handle)
	EndMethod
	
	Method transformMeshBuffer(buffer:IMeshBuffer, matrix:Matrix4)
		IrrScene_IMeshManipulator_transformMeshBuffer(handle, buffer.handle, matrix.handle)
	EndMethod

	Method createMeshWith1TCoords:IMesh(mesh:IMesh)
		Return IMesh.createFromHandle(IrrScene_IMeshManipulator_createMeshWith1TCoords(handle, _HANDLE(mesh)), True)
	EndMethod
	
	Method makePlanarTextureMappingFromBuffer(buffer:IMeshBuffer, resolution:Float = 0.001)
		IrrScene_IMeshManipulator_makePlanarTextureMappingFromBuffer(handle, _HANDLE(buffer), resolution)
	EndMethod
	
	Method makePlanarTextureMappingFromBufferWithOffset(buffer:IMeshBuffer, resolutionS:Float, resolutionT:Float, axis:Byte, offset:Vector3df)
		IrrScene_IMeshManipulator_makePlanarTextureMappingFromBufferWithOffset(handle, _HANDLE(buffer), resolutionS, resolutionT, axis, _HANDLE(offset))
	EndMethod

	Method recalculateTangents(mesh:IMesh, bRecalculateNormals:Int = False, bSmooth:Int = False, bAngleWeighted:Int = False)
		IrrScene_IMeshManipulator_recalculateTangents(handle, _HANDLE(mesh), bRecalculateNormals, bSmooth, bAngleWeighted)	
	EndMethod
	
	Method applyToMeshBuffer:Int(manipulator:IVertexManipulator, buffer:IMeshBuffer, bBoundingBoxUpdate:Int = False)
		Return IrrScene_IMeshManipulator_applyToMeshBuffer(handle, _HANDLE(manipulator), _HANDLE(buffer), bBoundingBoxUpdate)
	EndMethod
	
	Method applyToMesh:Int(manipulator:IVertexManipulator, mesh:IMesh, bBoundingBoxUpdate:Int = False)
		Return IrrScene_IMeshManipulator_applyToMesh(handle, _HANDLE(manipulator), _HANDLE(mesh), bBoundingBoxUpdate)
	EndMethod
EndType



