' ------------------------------------------------------------------------------------
'  irr::scene::ISkinnedMesh wrappers
' ------------------------------------------------------------------------------------
Public

Type ISkinnedMesh Extends IAnimatedMesh Final

	Method getJointCount:Int()
		Return IrrScene_ISkinnedMesh_getJointCount(handle)
	EndMethod
	
	Method getJointName:String(number:Int)
		Return IrrScene_ISkinnedMesh_getJointName(handle, number)
	EndMethod
	
	Method getJointNumber:Int(name:String)
		Return IrrScene_ISkinnedMesh_getJointNumber(handle, name)
	EndMethod
	
	Method useAnimationFrom:Int(other:ISkinnedMesh)
		IrrScene_ISkinnedMesh_useAnimationFrom(handle, other.handle)
	EndMethod
	
	Method updateNormalsWhenAnimating(bOn:Int)
		IrrScene_ISkinnedMesh_updateNormalsWhenAnimating(handle, bOn)
	EndMethod
	
	Method setInterpolationMode(E_INTERPOLATION_MODE:Int)
		IrrScene_ISkinnedMesh_setInterpolationMode(handle, E_INTERPOLATION_MODE)
	EndMethod
	
	Method animateMesh(frame:Float, blend:Float)
		IrrScene_ISkinnedMesh_animateMesh(handle, frame, blend)
	EndMethod
	
	Method skinMesh()
		IrrScene_ISkinnedMesh_skinMesh(handle)
	EndMethod
	
	Method convertMeshToTangents()
		IrrScene_ISkinnedMesh_convertMeshToTangents(handle)
	EndMethod

	Method setHardwareSkinning:Int(bOn:Int)
		Return IrrScene_ISkinnedMesh_setHardwareSkinning(handle, bOn)
	EndMethod
	
	Method isStatic:Int()
		Return IrrScene_ISkinnedMesh_isStatic(handle)
	EndMethod

	Method finalize()
		IrrScene_ISkinnedMesh_finalize(handle)
	EndMethod
	
	Method addMeshBuffer:SSkinMeshBuffer()
		Return SSkinMeshBuffer.createFromHandle(IrrScene_ISkinnedMesh_addMeshBuffer(handle), False)
	EndMethod
	
	Method addJoint:SJoint(parent:SJoint = Null)
		Return SJoint.createFromHandle(IrrScene_ISkinnedMesh_addJoint(handle, _HANDLE(parent)), False)
	EndMethod
	
	Method addWeight:SWeight(joint:SJoint)
		Return SWeight.createFromHandle(IrrScene_ISkinnedMesh_addWeight(handle, _HANDLE(joint)), False)
	EndMethod
	
	Method addPositionKey:SPositionKey(joint:SJoint)
		Return SPositionKey.createFromHandle(IrrScene_ISkinnedMesh_addPositionKey(handle, _HANDLE(joint)), False)
	EndMethod
	
	Method addScaleKey:SScaleKey(joint:SJoint)
		Return SScaleKey.createFromHandle(IrrScene_ISkinnedMesh_addScaleKey(handle, _HANDLE(joint)), False)
	EndMethod
	
	Method addRotationKey:SRotationKey(joint:SJoint)
		Return SRotationKey.createFromHandle(IrrScene_ISkinnedMesh_addRotationKey(handle, _HANDLE(joint)), False)
	EndMethod
	
	Method getMeshBuffers:SSkinMeshBuffer[]()

		Local ptrs:Int Ptr	
		Local objs:SSkinMeshBuffer[] = Null				
		Local size:Int = IrrScene_ISkinnedMesh_getMeshBuffers(handle, Varptr(ptrs)) 

		If size>0 Then		
			objs=New SSkinMeshBuffer[size]

			For Local i:Int=0 Until size
				objs[i]=SSkinMeshBuffer.createFromHandle(ptrs[i], False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return objs	
	EndMethod
	
	Method getAllJoints:SJoint[]()
		Local ptrs:Int Ptr	
		Local objs:SJoint[] = Null				
		Local size:Int = IrrScene_ISkinnedMesh_getAllJoints(handle, Varptr(ptrs)) 

		If size>0 Then		
			objs=New SJoint[size]

			For Local i:Int=0 Until size
				objs[i]=SJoint.createFromHandle(ptrs[i], False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return objs			
	EndMethod

	Function createFromHandle:ISkinnedMesh(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:ISkinnedMesh=New ISkinnedMesh
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction
	
EndType

Type SPositionKey Extends IrrBase Final

	Method getFrame:Float()
		Return IrrScene_ISkinnedMesh_SPositionKey_getFrame(handle) 
	EndMethod
	
	Method setFrame(frame:Float)
		IrrScene_ISkinnedMesh_SPositionKey_setFrame(handle, frame) 
	EndMethod
	
	Method getPosition:Vector3df()
		Return Vector3df.createFromHandle(IrrScene_ISkinnedMesh_SPositionKey_getPosition(handle), False)
	EndMethod
	
	Method setPosition(position:Vector3df)
		IrrScene_ISkinnedMesh_SPositionKey_setPosition(handle, position.handle) 
	EndMethod

	Function createFromHandle:SPositionKey(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:SPositionKey=New SPositionKey
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Function create:SPositionKey()
		Return createFromHandle(IrrScene_ISkinnedMesh_SPositionKey_new(), True)
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrScene_ISkinnedMesh_SPositionKey_del(handle)
	EndMethod	
		
EndType

Type SScaleKey Extends IrrBase Final

	Method getFrame:Float()
		Return IrrScene_ISkinnedMesh_SScaleKey_getFrame(handle) 
	EndMethod
	
	Method setFrame(frame:Float)
		IrrScene_ISkinnedMesh_SScaleKey_setFrame(handle, frame) 
	EndMethod
	
	Method getScale:Vector3df()
		Return Vector3df.createFromHandle(IrrScene_ISkinnedMesh_SScaleKey_getScale(handle), False)
	EndMethod
	
	Method setScale(scale:Vector3df)
		IrrScene_ISkinnedMesh_SScaleKey_setScale(handle, scale.handle) 
	EndMethod

	Function createFromHandle:SScaleKey(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:SScaleKey=New SScaleKey
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Function create:SScaleKey()
		Return createFromHandle(IrrScene_ISkinnedMesh_SScaleKey_new(), True)
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrScene_ISkinnedMesh_SScaleKey_del(handle)
	EndMethod	
		
EndType

Type SRotationKey Extends IrrBase Final

	Method getFrame:Float()
		Return IrrScene_ISkinnedMesh_SRotationKey_getFrame(handle) 
	EndMethod
	
	Method setFrame(frame:Float)
		IrrScene_ISkinnedMesh_SRotationKey_setFrame(handle, frame) 
	EndMethod
	
	Method getRotation:Quaternion()
		Return Quaternion.createFromHandle(IrrScene_ISkinnedMesh_SRotationKey_getRotation(handle), False)
	EndMethod
	
	Method setRotation(rotation:Quaternion)
		IrrScene_ISkinnedMesh_SRotationKey_setRotation(handle, rotation.handle) 
	EndMethod

	Function createFromHandle:SRotationKey(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:SRotationKey=New SRotationKey
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Function create:SRotationKey()
		Return createFromHandle(IrrScene_ISkinnedMesh_SRotationKey_new(), True)
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrScene_ISkinnedMesh_SRotationKey_del(handle)
	EndMethod	
		
EndType

Type SWeight Extends IrrBase Final

	Method getBufferId:Short()
		Return IrrScene_ISkinnedMesh_SWeight_getBufferId(handle) 
	EndMethod
	
	Method setBufferId(buffer_id:Short)
		IrrScene_ISkinnedMesh_SWeight_setBufferId(handle, buffer_id) 
	EndMethod
	
	Method getVertexId:Int()
		Return IrrScene_ISkinnedMesh_SWeight_getVertexId(handle) 
	EndMethod
	
	Method setVertexId(vertex_id:Int)
		IrrScene_ISkinnedMesh_SWeight_setVertexId(handle, vertex_id)
	EndMethod
	
	Method getStrength:Float()
		Return IrrScene_ISkinnedMesh_SWeight_getStrength(handle) 
	EndMethod
	
	Method setStrength(strength:Float)
		IrrScene_ISkinnedMesh_SWeight_setStrength(handle, strength) 
	EndMethod

	Function createFromHandle:SWeight(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:SWeight=New SWeight
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Function create:SWeight()
		Return createFromHandle(IrrScene_ISkinnedMesh_SWeight_new(), True)
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrScene_ISkinnedMesh_SWeight_del(handle)
	EndMethod	
		
EndType

Type SJoint Extends IrrBase Final

	Method getName:String()
		Return IrrScene_ISkinnedMesh_SJoint_getName(handle) 
	EndMethod
	
	Method setName(name:String)
		IrrScene_ISkinnedMesh_SJoint_setName(handle, name) 
	EndMethod
	
	Method getLocalMatrix:Matrix4()
		Return Matrix4.createFromHandle(IrrScene_ISkinnedMesh_SJoint_getLocalMatrix(handle), True)
	EndMethod
	
	Method setLocalMatrix(LocalMatrix:Matrix4)
		IrrScene_ISkinnedMesh_SJoint_setLocalMatrix(handle, _HANDLE(LocalMatrix))
	EndMethod

	Method getGlobalMatrix:Matrix4()
		Return Matrix4.createFromHandle(IrrScene_ISkinnedMesh_SJoint_getGlobalMatrix(handle), True)
	EndMethod
	
	Method setGlobalMatrix(GlobalMatrix:Matrix4)
		IrrScene_ISkinnedMesh_SJoint_setGlobalMatrix(handle, _HANDLE(GlobalMatriX))
	EndMethod

	Method getGlobalAnimatedMatrix:Matrix4()
		Return Matrix4.createFromHandle(IrrScene_ISkinnedMesh_SJoint_getGlobalAnimatedMatrix(handle), True)
	EndMethod
	
	Method setGlobalAnimatedMatrix(GlobalAnimatedMatrix:Matrix4)
		IrrScene_ISkinnedMesh_SJoint_setGlobalAnimatedMatrix(handle, _HANDLE(GlobalAnimatedMatrix))
	EndMethod

	Method getLocalAnimatedMatrix:Matrix4()
		Return Matrix4.createFromHandle(IrrScene_ISkinnedMesh_SJoint_getLocalAnimatedMatrix(handle), True)
	EndMethod
	
	Method setLocalAnimatedMatrix(LocalAnimatedMatrix:Matrix4)
		IrrScene_ISkinnedMesh_SJoint_setLocalAnimatedMatrix(handle, _HANDLE(LocalAnimatedMatrix))
	EndMethod

	Method getGlobalInversedMatrix:Matrix4()
		Return Matrix4.createFromHandle(IrrScene_ISkinnedMesh_SJoint_getGlobalInversedMatrix(handle), True)
	EndMethod
	
	Method setGlobalInversedMatrix(GlobalInversedMatrix:Matrix4)
		IrrScene_ISkinnedMesh_SJoint_setGlobalInversedMatrix(handle, _HANDLE(GlobalInversedMatrix))
	EndMethod

	Method getAnimatedposition:Vector3df()
		Return Vector3df.createFromHandle(IrrScene_ISkinnedMesh_SJoint_getAnimatedposition(handle), True)
	EndMethod
	
	Method setAnimatedposition(Animatedposition:Vector3df)
		IrrScene_ISkinnedMesh_SJoint_setAnimatedposition(handle, _HANDLE(Animatedposition))
	EndMethod

	Method getAnimatedscale:Vector3df()
		Return Vector3df.createFromHandle(IrrScene_ISkinnedMesh_SJoint_getAnimatedscale(handle), True)
	EndMethod
	
	Method setAnimatedScale(Animatedscale:Vector3df)
		IrrScene_ISkinnedMesh_SJoint_setAnimatedscale(handle, _HANDLE(Animatedscale))
	EndMethod
	
	Method getAnimatedrotation:Quaternion()
		Return Quaternion.createFromHandle(IrrScene_ISkinnedMesh_SJoint_getAnimatedrotation(handle), True)
	EndMethod
	
	Method setAnimatedrotation(Animatedrotation:Quaternion)
		IrrScene_ISkinnedMesh_SJoint_setAnimatedrotation(handle, _HANDLE(Animatedrotation))
	EndMethod


	Method setPositionKeys(PositionKeys:SPositionKey[])
		Local positionkeys_size:Int = 0
		Local positionkeys_array:Int[] = GetHandleArray(PositionKeys)
		If positionkeys_array Then positionkeys_size = positionkeys_array.Dimensions()[0]	

		IrrScene_ISkinnedMesh_SJoint_setPositionKeys(handle, positionkeys_array, positionkeys_size)
	EndMethod
	
	Method getPositionKeys:SPositionKey[]()
		Local ptrs:Int Ptr	
		Local objs:SPositionKey[] = Null				

		Local size:Int = IrrScene_ISkinnedMesh_SJoint_getPositionKeys(handle, Varptr(ptrs))

		If size>0 Then		
			objs=New SPositionKey[size]

			For Local i:Int=0 Until size
				objs[i]=SPositionKey.createFromHandle(ptrs[i],False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return objs		
	EndMethod

	Method setScaleKeys(ScaleKeys:SScaleKey[])
		Local scalekeys_size:Int = 0
		Local scalekeys_array:Int[] = GetHandleArray(ScaleKeys)
		If scalekeys_array Then scalekeys_size = scalekeys_array.Dimensions()[0]	

		IrrScene_ISkinnedMesh_SJoint_setScaleKeys(handle, scalekeys_array, scalekeys_size)
	EndMethod

	Method getScaleKeys:SScaleKey[]()
		Local ptrs:Int Ptr	
		Local objs:SScaleKey[] = Null				

		Local size:Int = IrrScene_ISkinnedMesh_SJoint_getScaleKeys(handle, Varptr(ptrs))

		If size>0 Then		
			objs=New SScaleKey[size]

			For Local i:Int=0 Until size
				objs[i]=SScaleKey.createFromHandle(ptrs[i],False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return objs		
	EndMethod

	Method setRotationKeys(RotationKeys:SRotationKey[])
		Local keys_size:Int = 0
		Local keys_array:Int[] = GetHandleArray(RotationKeys)
		If keys_array Then keys_size = keys_array.Dimensions()[0]	

		IrrScene_ISkinnedMesh_SJoint_setRotationKeys(handle, keys_array, keys_size)
	EndMethod

	Method getRotationKeys:SRotationKey[]()
		Local ptrs:Int Ptr	
		Local objs:SRotationKey[] = Null				

		Local size:Int = IrrScene_ISkinnedMesh_SJoint_getRotationKeys(handle, Varptr(ptrs))

		If size>0 Then		
			objs=New SRotationKey[size]

			For Local i:Int=0 Until size
				objs[i]=SRotationKey.createFromHandle(ptrs[i],False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return objs		
	EndMethod

	Method setWeights(Weights:SWeight[])
		Local weights_size:Int = 0
		Local weights_array:Int[] = GetHandleArray(Weights)
		If weights_array Then weights_size = weights_array.Dimensions()[0]	

		IrrScene_ISkinnedMesh_SJoint_setWeights(handle, weights_array, weights_size)
	EndMethod

	Method getWeights:SWeight[]()
		Local ptrs:Int Ptr	
		Local objs:SWeight[] = Null				

		Local size:Int = IrrScene_ISkinnedMesh_SJoint_getWeights(handle, Varptr(ptrs))

		If size>0 Then		
			objs=New SWeight[size]

			For Local i:Int=0 Until size
				objs[i]=SWeight.createFromHandle(ptrs[i],False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return objs		
	EndMethod

	Method setChildren(children:SJoint[])
		Local children_size:Int = 0
		Local children_array:Int[] = GetHandleArray(children)
		If children_array Then children_size = children_array.Dimensions()[0]	

		IrrScene_ISkinnedMesh_SJoint_setChildren(handle, children_array, children_size)
	EndMethod
	
	Method getChildren:SJoint[]()
		Local ptrs:Int Ptr	
		Local objs:SJoint[] = Null				

		Local size:Int = IrrScene_ISkinnedMesh_SJoint_getChildren(handle, Varptr(ptrs))

		If size>0 Then		
			objs=New SJoint[size]

			For Local i:Int=0 Until size
				objs[i]=SJoint.createFromHandle(ptrs[i],False)
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return objs		
	EndMethod

	Method setAttachedMeshes(AttachedMeshes:Int[])
		Local size:Int = 0
		If AttachedMeshes Then size = AttachedMeshes.Dimensions()[0]	

		IrrScene_ISkinnedMesh_SJoint_setAttachedMeshes(handle, AttachedMeshes, size)
	EndMethod

	Method getAttachedMeshes:Int[]()
		Local ptrs:Int Ptr	
		Local meshes:Int[] = Null				

		Local size:Int = IrrScene_ISkinnedMesh_SJoint_getAttachedMeshes(handle, Varptr(ptrs))

		If size>0 Then		
			meshes = New Int[size]

			For Local i:Int=0 Until size
				meshes[i] = ptrs[i]
			Next
		EndIf
		
		Irr_ptrarray_delete(Varptr(ptrs))
		Return meshes		
	EndMethod

	Function createFromHandle:SJoint(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:SJoint=New SJoint
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

	Function create:SJoint()
		Return createFromHandle(IrrScene_ISkinnedMesh_SJoint_new(), True)
	EndFunction

	Method Delete()
		If (mustdelete) Then IrrScene_ISkinnedMesh_SJoint_del(handle)
	EndMethod	
	
EndType
