
Public

Type IGeometryCreator Extends IReferenceCounted Final


	Method createCubeMesh:IMesh(size:Vector3df = Null)
		If Not size Then size = _VECTOR3DF(5.0,5.0,5.0)
		Return IMesh.createFromHandle(IrrVideo_IGeometryCreator_createCubeMesh(handle, _HANDLE(size)), True)
	EndMethod
	
	Method createHillPlaneMesh:IMesh(tileSize:Dimension2df, tileCount:Dimension2di, material:SMaterial, hillHeight:Float, countHills:Dimension2df, textureRepeatCount:Dimension2df)
		Return IMesh.createFromHandle(IrrVideo_IGeometryCreator_createHillPlaneMesh(handle, _HANDLE(tileSize), _HANDLE(tileCount), _HANDLE(material), hillHeight, _HANDLE(countHills), _HANDLE(textureRepeatCount)), True)
	EndMethod
	
	Method createPlaneMesh:IMesh(tileSize:Dimension2df, tileCount:Dimension2di, material:SMaterial, textureRepeatCount:Dimension2df)
		Return IMesh.createFromHandle(IrrVideo_IGeometryCreator_createPlaneMesh(handle, _HANDLE(tileSize), _HANDLE(tileCount), _HANDLE(material), _HANDLE(textureRepeatCount)), True)
	EndMethod

	Method createTerrainMesh:IMesh(texture:IImage, heightmap:IImage, stretchSize:Dimension2df, maxHeight:Float, driver:IVideoDriver, defaultVertexBlockSize:Dimension2di, bDebugBorders:Int = False)
		Return IMesh.createFromHandle(IrrVideo_IGeometryCreator_createTerrainMesh(handle, _HANDLE(texture), _HANDLE(heightmap), _HANDLE(stretchSize), maxHeight, _HANDLE(driver), _HANDLE(defaultVertexBlockSize), bDebugBorders), True)
	EndMethod
	
	Method createArrowMesh:IMesh(tesselationCylinder:Int = 4, tesselationCone:Int = 8, height:Float = 1.0, cylinderHeight:Float = 0.6, widthCylinder:Float = 0.05, widthCone:Float = 0.3, colorCylinder:SColor = Null, colorCone:SColor = Null)
		If Not colorCylinder Then colorCylinder = SColor.createFromColorVal($ffffffff)
		If Not colorCone Then colorCone = SColor.createFromColorVal($ffffffff)
		Return IMesh.createFromHandle(IrrVideo_IGeometryCreator_createArrowMesh(handle, tesselationCylinder, tesselationCone, height, cylinderHeight, widthCylinder, widthCone, _HANDLE(colorCylinder), _HANDLE(colorCone)), True)
	EndMethod
	
	Method createSphereMesh:IMesh(radius:Float = 5.0, polyCountX:Int = 16, polyCountY:Int = 16)
		Return IMesh.createFromHandle(IrrVideo_IGeometryCreator_createSphereMesh(handle, radius, polyCountX, polyCountY), True)
	EndMethod
	
	Method createCylinderMesh:IMesh(radius:Float, length:Float, tesselation:Int, color:SColor = Null, bCloseTop:Int = True, oblique:Float = 0.0)
		If Not color Then color = SColor.createFromColorVal($ffffffff)
		Return IMesh.createFromHandle(IrrVideo_IGeometryCreator_createCylinderMesh(handle, radius, length, tesselation, _HANDLE(color), bCloseTop, oblique), True)
	EndMethod
	
	Method createConeMesh:IMesh(radius:Float, length:Float, tesselation:Int, colorTop:SColor = Null, colorBottom:SColor = Null, oblique:Float = 0.0)
		If Not colorTop Then colorTop = SColor.createFromColorVal($ffffffff)
		If Not colorBottom Then colorBottom = SColor.createFromColorVal($ffffffff)
		Return IMesh.createFromHandle(IrrVideo_IGeometryCreator_createConeMesh(handle, radius, length, tesselation, _HANDLE(colorTop), _HANDLE(colorBottom), oblique), True)
	EndMethod
	
	Method createVolumeLightMesh:IMesh(subdivideU:Int = 32, subdivideV:Int = 32, footColor:SColor = Null, tailColor:SColor = Null, lpDistance:Float = 8.0, lightDim:Vector3df = Null)
		If Not footColor Then footColor = SColor.createFromColorVal($ffffffff)
		If Not tailColor Then tailColor = SColor.createFromColorVal($ffffffff)
		If Not lightDim Then lightDim = _VECTOR3DF(1.0, 1.2, 1.0)
	
		Return IMesh.createFromHandle(IrrVideo_IGeometryCreator_createVolumeLightMesh(handle, subdivideU, subdivideV, _HANDLE(footColor), _HANDLE(tailColor), lpDistance, _HANDLE(lightDim)), True)
	EndMethod

	Function createFromHandle:IGeometryCreator(pHandle:Int,bMustDel:Int=False)
		If Not pHandle Then Return Null
		Local oRetVal:IGeometryCreator=New IGeometryCreator
		oRetVal.attach(pHandle,bMustDel)
		Return oRetVal
	EndFunction

 	Method drop:Int() Final
		If mustdelete Then mustdelete=False		
		Return IrrVideo_IGeometryCreator_drop(handle)
	EndMethod
	
	Method getDebugName:String() Final
		Return IrrVideo_IGeometryCreator_getDebugName(handle)
	EndMethod
	
	Method grab() Final
		IrrVideo_IGeometryCreator_grab(handle)
	EndMethod

	Method Delete()
		If (mustdelete) Then drop()
	EndMethod	
EndType
