Rem
* Example 004 Movement

This Tutorial shows how To move And Animate SceneNodes. The
basic concept of SceneNodeAnimators is shown as well as manual
movement of nodes using the keyboard.  We'll demonstrate framerate
independent movement, which means moving by an amount dependent
on the duration of the last run of the Irrlicht loop.

Example 19.MouseAndJoystick shows how To handle those kinds of Input.

As always, I Include the header files, use the irr namespace,
And tell the linker To link with the .lib file.
EndRem
SuperStrict
Framework Irrlicht.Core

Rem
To receive events like mouse And keyboard Input, Or GUI events like "the OK
button has been clicked", we need an object which is derived from the
irr::IEventReceiver Object. There is only one Method To override:
irr::IEventReceiver::OnEvent(). This Method will be called by the engine once
when an event happens. What we really want To know is whether a key is being
held down, And so we will remember the current state of each key.
EndRem
Type MyEventReceiver Extends IEventReceiver
	Field KeyIsDown:Int[EKEY_KEY_CODES_COUNT]

	' This is the one Method that we have To implement
	Method OnEvent:Int(event:SEvent)
		' Remember whether each key is down Or up
		If event.getEventType() = EET_KEY_INPUT_EVENT
			KeyIsDown[event.getKeyInputKey()] = event.getKeyPressedDown()
		EndIf

		Return False
	EndMethod

	' This is used To check whether a key is being held down
	Method IsKeyDown:Int(EKEY_CODE:Int)
		Return KeyIsDown[EKEY_CODE]
	EndMethod
	
	Method New()
		For Local i:Int = 0 Until EKEY_KEY_CODES_COUNT
			KeyIsDown[i] = False
		Next
	EndMethod

	' we must override the generate function in order for instantiation to work properly
	Function generate:IEventReceiver()
		Return New MyEventReceiver
	EndFunction
EndType

Rem
The event receiver For keeping the pressed keys is ready, the actual responses
will be made inside the render loop, Right before drawing the scene. So lets
just Create an irr::IrrlichtDevice And the scene node we want To move. We also
Create some other additional scene nodes, To show that there are also some
different possibilities To move And Animate scene nodes.
EndRem

' let user Select driver Type

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' Create device

Local receiver:MyEventReceiver = ..
	MyEventReceiver(IEventReceiver.Create(MyEventReceiver.generate))

Local device:IrrlichtDevice = IrrlichtDevice.Create(driverType, _DIMENSION2DI(640, 480), ..
	16,False,False,False,receiver)

If Not device Return ' could Not create selected driver.

Local driver:IVideoDriver = device.getVideoDriver()
Local smgr:ISceneManager = device.getSceneManager()

Rem
Create the node which will be moved with the WSAD keys. We Create a
sphere node, which is a built-in geometry primitive. We place the node
at (0,0,30) And assign a texture To it To let it look a little bit more
interesting. Because we have no dynamic lights in this scene we disable
lighting For each model (otherwise the models would be black).
EndRem
Local node:ISceneNode = smgr.addSphereSceneNode()
If node 
	node.setPosition(_VECTOR3DF(0,0,30))
	node.setMaterialTexture(0,driver.getTexture("../media/wall.bmp"))
	node.setMaterialFlag(EMF_LIGHTING,False)
EndIf
	
Rem
Now we Create another node, movable using a scene node animator. Scene
node animators modify scene nodes And can be attached To any scene node
like mesh scene nodes, billboards, lights And even camera scene nodes.
Scene node animators are Not only able To modify the position of a
scene node, they can also Animate the textures of an Object For
example. We Create a cube scene node And attach a 'fly circle' scene
node animator To it, letting this node fly around our sphere scene node.
EndRem
Local n:ISceneNode = smgr.addCubeSceneNode()

If n
	n.setMaterialTexture(0, driver.getTexture("../media/t351sml.jpg"))
	n.setMaterialFlag(EMF_LIGHTING, False)
	Local anim:ISceneNodeAnimator = ..
		smgr.createFlyCircleAnimator(_VECTOR3DF(0,0,30), 20.0)
	n.addAnimator(anim)
	anim.drop()
EndIf

For Local i:Int = 0 Until 200
	smgr.addSphereSceneNode()
Next

Rem
The last scene node we add To show possibilities of scene node animators is
a b3d model, which uses a 'fly straight' animator to run between to points.
EndRem
Local anms:IAnimatedMeshSceneNode = ..
	smgr.addAnimatedMeshSceneNode(smgr.getMesh("../media/ninja.b3d"))

If anms

	Local anim:ISceneNodeAnimator = ..
		smgr.createFlyStraightAnimator(_VECTOR3DF(100,0,60), ..
		_VECTOR3DF(-100,0,60), 3500, True)
	anms.addAnimator(anim)
	anim.drop()
	
	Rem
	To make the model look Right we disable lighting, set the
	frames between which the animation should loop, rotate the
	model around 180 degrees, And adjust the animation speed And
	the texture. To set the Right animation (frames And speed), we
	would also be able To just call
	"anms->setMD2Animation(scene::EMAT_RUN)" For the 'run'
	animation instead of "setFrameLoop" And "setAnimationSpeed",
	but this only works with MD2 animations, And so you know how To
	start other animations. But a good advice is To Not use
	hardcoded frame-numbers...
	EndRem
	anms.setMaterialFlag(EMF_LIGHTING, False)

	anms.setFrameLoop(0, 13)
	anms.setAnimationSpeed(15)

	anms.SetScale(_VECTOR3DF(2,2,2))
	anms.SetRotation(_VECTOR3DF(0,-90,0))
EndIf

Rem
To be able To look at And move around in this scene, we Create a first
person shooter style camera And make the mouse cursor invisible.
EndRem
Local cam:ICameraSceneNode = smgr.addCameraSceneNodeFPS()
device.getCursorControl().setVisible(False)

Rem
Add a colorful irrlicht logo
EndRem
device.getGUIEnvironment().addImageFromTexture( ..
	driver.getTexture("../media/irrlichtlogoalpha2.tga"), ..
	_POSITION2DI(10,20))

Local diagnostics:IGUIStaticText = device.getGUIEnvironment().addStaticText( ..
	"", _RECTI(10, 10, 400, 20))
diagnostics.setOverrideColor(_SCOLOR(255, 255, 255, 0))

Rem
We have done everything, so lets draw it. We also write the current
frames per second And the name of the driver To the caption of the
window.
EndRem
Local lastFPS:Int = -1

' In order To do framerate independent movement, we have To know
' how Long it was since the last frame
Local lastTime:Int = device.getTimer().getTime()

' This is the movemen speed in units per second.
Const MOVEMENT_SPEED:Float = 5.0


While device.run()

	' Work out a frame delta time.
	Local now:Int = device.getTimer().getTime()
	Local frameDeltaTime:Float = Float(now - lastTime) / 1000  ' Time in seconds
	lastTime = now
	
	Rem 
	Check If keys W, S, A Or D are being held down, And move the
	sphere node around respectively. 
	EndRem
	Local nodePosition:Vector3df = node.getPosition()

	If receiver.IsKeyDown(EKEY_KEY_W)
		nodePosition.setY(nodePosition.getY() + (MOVEMENT_SPEED * frameDeltaTime))
	ElseIf receiver.IsKeyDown(EKEY_KEY_S)
		nodePosition.setY(nodePosition.getY() - (MOVEMENT_SPEED * frameDeltaTime))
	EndIf

	If receiver.IsKeyDown(EKEY_KEY_A)
		nodePosition.setX(nodePosition.getX() - (MOVEMENT_SPEED * frameDeltaTime))
	ElseIf receiver.IsKeyDown(EKEY_KEY_D)
		nodePosition.setX(nodePosition.getX() + (MOVEMENT_SPEED * frameDeltaTime))
	EndIf

	node.setPosition(nodePosition)

	driver.beginScene(True, True, _SCOLOR(255,113,113,133))

	smgr.drawAll() ' draw the 3d scene
	device.getGUIEnvironment().drawAll()  ' draw the gui environment (the logo)

	driver.endScene()

	Local fps:Int = driver.getFPS()

	If (lastFPS <> fps)
		Local tmp:String = "Movement Example - Irrlicht Engine ["
		tmp :+ driver.getName()
		tmp :+ "] fps: " 
		tmp :+ fps

		device.setWindowCaption(tmp)
		lastFPS = fps
	EndIf
Wend

Rem
In the End, Delete the Irrlicht device.
EndRem
device.drop()

Rem
That's it. Compile and play around with the program.
EndRem
