Rem
* Example 011 Per-Pixel Lighting

This tutorial shows how To use one of the built in more complex materials in
irrlicht: Per pixel lighted surfaces using normal maps And parallax mapping. It
will also show how To use fog And moving particle systems. And don't panic: You
dont need any experience with shaders To use these materials in Irrlicht.

At first, we need To Include all headers And do the stuff we always do, like in
nearly all other tutorials.
EndRem
SuperStrict
Framework Irrlicht.Core

Rem
For this example, we need an event receiver, To make it possible For the user
To switch between the three available material types. In addition, the event
receiver will Create some small GUI window which displays what material is
currently being used. There is nothing special done in this class, so maybe you
want To skip reading it.
EndRem

Type MyEventReceiver Extends IEventReceiver
	
	Method Init:MyEventReceiver(roomnode:ISceneNode, ..
		env:IGUIEnvironment,drv:IVideoDriver)
		
		' store pointer To room so we can change its drawing mode
		Room = roomnode
		Driver = drv

		' set a nicer font
		Local skin:IGUISkin = env.getSkin()
		Local font:IGUIFont = env.getFont("../media/fonthaettenschweiler.bmp")
		If (font) Then ..
			skin.setFont(font)

		' add window And listbox
		Local window:IGUIWindow = env.addWindow( ..
			_RECTI(460,375,630,470), False, "Use 'E' + 'R' to change")

		ListBox = env.addListBox( ..
			_RECTI(2,22,165,88),window)

		ListBox.addItem("Diffuse")
		ListBox.addItem("Bump mapping")
		ListBox.addItem("Parallax mapping")
		ListBox.setSelected(1)

		' create problem text
		ProblemText = env.addStaticText( ..
			"Your hardware or this renderer is not able to use the "+ ..
			"needed shaders for this material. Using fall back materials.", ..
			_RECTI(150,20,470,80))

		ProblemText.setOverrideColor(_SCOLOR(100,255,255,255))

		' set start material (prefer parallax mapping If available)
		Local renderer:IMaterialRenderer = ..
			Driver.getMaterialRenderer(EMT_PARALLAX_MAP_SOLID)
			
		If (renderer And renderer.getRenderCapability() = 0)
			ListBox.setSelected(2)
		EndIf
		
		' set the material which is selected in the listbox
		setMaterial()
		
		Return Self
	EndMethod

	Method OnEvent:Int(event:SEvent)
		' check If user presses the key 'E' or 'R'
		If (event.getEventType()=EET_KEY_INPUT_EVENT And ..
			Not event.getKeyPressedDown() And 	Room And ListBox)

			' change selected item in listbox

			Local sel:Int = ListBox.getSelected()
			Local key:Int=event.getKeyInputKey()
					
			If (key = EKEY_KEY_R)
				sel:+1
			Else
				If (key = EKEY_KEY_E)
					sel:-1
				Else 
					Return False
				EndIf
			EndIf

			If (sel > 2) Then sel = 0
			If (sel < 0) Then sel = 2
			ListBox.setSelected(sel)
			
			' set the material which is selected in the listbox
			setMaterial()
		EndIf

		Return False
	EndMethod

	' sets the material of the room mesh the the one set in the 
	' list box.
	Method setMaterial()
		Local mattype:Int = EMT_SOLID

		' change material setting
		Select (ListBox.getSelected())
			Case 0 mattype = EMT_SOLID
			Case 1 mattype = EMT_NORMAL_MAP_SOLID
			Case 2 mattype = EMT_PARALLAX_MAP_SOLID
		EndSelect

		Room.setMaterialType(mattype)

		Rem
		We need To add a warning If the materials will Not be able To
		be displayed 100% correctly. This is no problem, they will be
		renderered using fall back materials, but at least the user
		should know that it would look better on better hardware. We
		simply check If the material renderer is able To draw at full
		quality on the current hardware. The
		IMaterialRenderer::getRenderCapability() returns 0 If this is
		the Case.
		EndRem
		Local renderer:IMaterialRenderer = Driver.getMaterialRenderer(mattype)

		' display some problem text when problem
		If (Not renderer Or renderer.getRenderCapability() <> 0)
			ProblemText.setVisible(True)
		Else
			ProblemText.setVisible(False)
		EndIf	

	EndMethod

	Field ProblemText:IGUIStaticText
	Field ListBox:IGUIListBox
	Field Room:ISceneNode
	Field Driver:IVideoDriver

	' we must override the generate function in order for instantiation to work properly
	Function generate:IEventReceiver()
		Return New MyEventReceiver
	EndFunction
EndType

Rem
Now For the real fun. We Create an Irrlicht Device And start To setup the scene.
EndRem

' let user Select driver Type

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' create device And Exit If creation failed

Local device:IrrlichtDevice = ..
	IrrlichtDevice.Create(driverType, _DIMENSION2DI(640, 480))

If Not device Return ' could Not create selected driver.

Rem
Before we start with the interesting stuff, we do some simple things:
Store pointers To the most important parts of the engine (video driver,
scene manager, gui environment) To safe us from typing too much, add an
irrlicht engine logo To the window And a user controlled first person
shooter style camera. Also, we let the engine know that it should store
all textures in 32 bit. This necessary because For parallax mapping, we
need 32 bit textures.
EndRem

Local driver:IVideoDriver=device.getVideoDriver()
Local smgr:ISceneManager=device.getSceneManager()
Local env:IGUIEnvironment=device.getGUIEnvironment()

driver.setTextureCreationFlag(ETCF_ALWAYS_32_BIT, True)

'  add irrlicht logo
env.addImageFromTexture(driver.getTexture("../media/irrlichtlogo2.png"), ..
	_POSITION2DI(10,10))
	
'  add camera
Local camera:ICameraSceneNode = smgr.addCameraSceneNodeFPS()
camera.setPosition(_VECTOR3DF(-200,200,-200))

'  disable mouse cursor
device.getCursorControl().setVisible(False)

Rem
Because we want the whole scene To look a little bit scarier, we add
some fog To it. This is done by a call To IVideoDriver::setFog(). There
you can set various fog settings. In this example, we use pixel fog,
because it will work well with the materials we'll use in this example.
Please note that you will have To set the material flag EMF_FOG_ENABLE
To 'true' in every scene node which should be affected by this fog.
EndRem
driver.setFog(_SCOLOR(0,138,125,81), EFT_FOG_LINEAR, 250, 1000, .003, True, False)

Rem
To be able To display something interesting, we Load a mesh from a .3ds
file which is a room I modeled with anim8or. It is the same room as
from the specialFX example. Maybe you remember from that tutorial, I am
no good modeler at all And so I totally messed up the texture mapping
in this model, but we can simply repair it with the
IMeshManipulator::makePlanarTextureMapping() Method.
EndRem

Local roomMesh:IAnimatedMesh = smgr.getMesh( ..
	"../media/room.3ds")
Local room:ISceneNode

If (roomMesh)

	smgr.getMeshManipulator().makePlanarTextureMapping( ..
			roomMesh.getMesh(0), 0.003)

	Rem
	Now For the first exciting thing: If we successfully loaded the
	mesh we need To apply textures To it. Because we want this room
	To be displayed with a very cool material, we have To do a
	little bit more than just set the textures. Instead of only
	loading a color map as usual, we also Load a height map which
	is simply a grayscale texture. From this height map, we Create
	a normal map which we will set as second texture of the room.
	If you already have a normal map, you could directly set it,
	but I simply didn't find a nice normal map for this texture.
	The normal map texture is being generated by the
	makeNormalMapTexture Method of the VideoDriver. The second
	parameter specifies the height of the heightmap. If you set it
	To a bigger value, the map will look more rocky.
	EndRem

	Local normalMap:ITexture = ..
		driver.getTexture("../media/rockwall_height.bmp")
	
	driver.makeNormalMapTexture(normalMap, 9.0)
	If (normalMap) Then driver.makeNormalMapTexture(normalMap, 9.0)

	Rem
	But just setting color And normal map is Not everything. The
	material we want To use needs some additional informations per
	vertex like tangents And binormals. Because we are too lazy To
	calculate that information now, we let Irrlicht do this For us.
	That's why we call IMeshManipulator::createMeshWithTangents().
	It creates a mesh copy with tangents And binormals from another
	mesh. After we've done that, we simply create a standard
	mesh scene node with this mesh copy, set color And normal map
	And adjust some other material settings. Note that we set
	EMF_FOG_ENABLE To True To enable fog in the room.
	EndRem

	Local tangentMesh:IMesh = smgr.getMeshManipulator().createMeshWithTangents( ..
		roomMesh.getMesh(0))

	room = smgr.addMeshSceneNode(tangentMesh)
	room.setMaterialTexture(0, driver.getTexture("../media/rockwall.jpg"))
	room.setMaterialTexture(1, normalMap)

	room.getMaterial(0).getSpecularColor().set(0,0,0,0)

	room.setMaterialFlag(EMF_FOG_ENABLE, True)
	room.setMaterialType(EMT_PARALLAX_MAP_SOLID) 
	room.getMaterial(0).setMaterialTypeParam(0.035) '  adjust height For parallax effect

	'  drop mesh because we created it with a create.. call.
	tangentMesh.drop()
EndIf

Rem
After we've created a room shaded by per pixel lighting, we add a
sphere into it with the same material, but we'll make it transparent.
In addition, because the sphere looks somehow like a familiar planet,
we make it rotate. The procedure is similar as before. The difference
is that we are loading the mesh from an .x file which already contains
a color map so we do Not need To Load it manually. But the sphere is a
little bit too small For our needs, so we scale it by the factor 50.
EndRem

' add earth sphere

Local earthMesh:IAnimatedMesh = smgr.getMesh("../media/earth.x")
If (earthMesh)

	' perform various task with the mesh manipulator
	Local manipulator:IMeshManipulator = smgr.getMeshManipulator()

	'  create mesh copy with tangent informations from original earth.x mesh
	Local tangentSphereMesh:IMesh = ..
		manipulator.createMeshWithTangents(earthMesh.getMesh(0))

	'  set the alpha value of all vertices To 200
	manipulator.setVertexColorAlpha(tangentSphereMesh, 200)

	' scale the mesh by factor 50
	Local m:matrix4 = Matrix4.Create()
	m.SetScale ( _VECTOR3DF(50,50,50) )
	manipulator.transformMesh( tangentSphereMesh, m )

	Local sphere:ISceneNode = smgr.addMeshSceneNode(tangentSphereMesh)

	sphere.setPosition(_VECTOR3DF(-70,130,45))

	'  load heightmap, create normal map from it And set it
	Local earthNormalMap:ITexture = driver.getTexture("../media/earthbump.bmp")
	If (earthNormalMap)
		driver.makeNormalMapTexture(earthNormalMap, 20.0)
		sphere.setMaterialTexture(1, earthNormalMap)
		sphere.setMaterialType(EMT_NORMAL_MAP_TRANSPARENT_VERTEX_ALPHA)
	EndIf

	'  adjust material settings
	sphere.setMaterialFlag(EMF_FOG_ENABLE, True)

	' add rotation animator
	Local anim:ISceneNodeAnimator = ..
		smgr.createRotationAnimator(_VECTOR3DF(0,0.1,0))	
	sphere.addAnimator(anim)
	anim.drop()

	' drop mesh because we created it with a Create.. call.
	tangentSphereMesh.drop()
EndIf

Rem
Per pixel lighted materials only look cool when there are moving
lights. So we add some. And because moving lights alone are so boring,
we add billboards To them, And a whole particle system To one of them.
We start with the first light which is red And has only the billboard
attached.
EndRem

'  add light 1 (nearly red)
Local light1:ILightSceneNode = ..
	smgr.addLightSceneNode(Null, _VECTOR3DF(0,0,0), ..
	_SCOLORF(0.5, 1.0, 0.5, 0.0), 800.0)

light1.setDebugDataVisible ( EDS_BBOX )

'  add fly circle animator To light 1
Local anim:ISceneNodeAnimator = ..
	smgr.createFlyCircleAnimator (_VECTOR3DF(50,300,0),190.0, -0.003)
light1.addAnimator(anim)
anim.drop()

'  attach billboard To the light
Local bill:ISceneNode = ..
	smgr.addBillboardSceneNode(light1, _DIMENSION2DF(60, 60))

bill.setMaterialFlag(EMF_LIGHTING, False)
bill.setMaterialFlag(EMF_ZWRITE_ENABLE, False)
bill.setMaterialType(EMT_TRANSPARENT_ADD_COLOR)
bill.setMaterialTexture(0, driver.getTexture("../media/particlered.bmp"))

Rem
Now the same again, with the second light. The difference is that we
add a particle system To it too. And because the light moves, the
particles of the particlesystem will follow. If you want To know more
about how particle systems are created in Irrlicht, take a look at the
specialFx example. Maybe you will have noticed that we only add 2
lights, this has a simple reason: The low End version of this material
was written in ps1.1 And vs1.1, which doesn't allow more lights. You
could add a third light To the scene, but it won't be used to shade the
walls. But of course, this will change in future versions of Irrlicht
where higher versions of pixel/vertex shaders will be implemented too.
EndRem

'  add light 2 (gray)
Local light2:ISceneNode = ..
	smgr.addLightSceneNode(Null, _VECTOR3DF(0,0,0), ..
	_SCOLORF(1.0, 0.2, 0.2, 0.0), 800.0)

'  add fly circle animator To light 2
anim = smgr.createFlyCircleAnimator (_VECTOR3DF(0,150,0),200.0, ..
	0.001, _VECTOR3DF( 0.2, 0.9, 0.0 )) 
light2.addAnimator(anim)
anim.drop()

'  attach billboard To light
bill = smgr.addBillboardSceneNode(light2, _DIMENSION2DF(120, 120))
bill.setMaterialFlag(EMF_LIGHTING, False)
bill.setMaterialFlag(EMF_ZWRITE_ENABLE, False)
bill.setMaterialType(EMT_TRANSPARENT_ADD_COLOR)
bill.setMaterialTexture(0, driver.getTexture("../media/particlewhite.bmp"))

'  add particle system
Local ps:IParticleSystemSceneNode = ..
	smgr.addParticleSystemSceneNode(False, light2)

'  create And set emitter
Local em:IParticleEmitter = ps.createBoxEmitter( ..
	AABBox3df.createFromVals(-3,0,-3,3,1,3), ..
	_VECTOR3DF(0.0,0.03,0.0), ..
	80,100, ..
	_SCOLOR(0,255,255,255), _SCOLOR(0,255,255,255), ..
	400,1100)
em.setMinStartSize(_DIMENSION2DF(30.0, 40.0))
em.setMaxStartSize(_DIMENSION2DF(30.0, 40.0))

ps.setEmitter(em)
em.drop()

'  create And set affector
Local paf:IParticleAffector = ps.createFadeOutParticleAffector()
ps.addAffector(paf)
paf.drop()

'  adjust some material settings
ps.setMaterialFlag(EMF_LIGHTING, False)
ps.setMaterialFlag(EMF_ZWRITE_ENABLE, False)
ps.setMaterialTexture(0, driver.getTexture("../media/fireball.bmp"))
ps.setMaterialType(EMT_TRANSPARENT_VERTEX_ALPHA)

Local receiver:MyEventReceiver = ..
	MyEventReceiver(MyEventReceiver.Create(MyEventReceiver.generate)).Init(room, env, driver)
device.setEventReceiver(receiver)
	
Rem
Finally, draw everything. That's it.
EndRem

Local lastFPS:Int = -1

While(device.run())
	If (device.isWindowActive())
	
		driver.beginScene(True, True, Null)
	
		smgr.drawAll()
		env.drawAll()
	
		driver.endScene()
	
		Local fps:Int = driver.getFPS()
	
		If (lastFPS <> fps)
			Local str:String = "Per pixel lighting example - Irrlicht Engine ["
			str :+ driver.getName()
			str :+ "] FPS:"
			str :+ fps
			
			device.setWindowCaption(str)
			lastFPS = fps
		EndIf
	EndIf
Wend

device.drop()
