Rem* Example 014 Win32 Window

This example only runs under MS Windows And demonstrates that Irrlicht can
render inside a win32 window. MFC And .NET Windows.Forms windows are possible,
too.

In the begining, we Create a windows window using the windows API. I'm not
going To explain this code, because it is windows specific. See the MSDN Or a
windows book For details.
EndRem

SuperStrict 
Framework Irrlicht.Core

Import BRL.Basic
Import MaxGUI.Win32MaxGUIEx
Import BRL.EventQueue

Local window:TGadget
Local panel:TGadget

Local windowWidth:Int = 440
Local windowHeight:Int = 380

' the original does not have this.  this is just to show that you can mix 
' events between MaxGUI And Irrlicht.
Type MyEventReceiver Extends IEventReceiver

	Field box:ISceneNode

	Method setBox(b:ISceneNode)
		box=b
	EndMethod

	Method OnEvent:Int(event:SEvent)

		' check If user presses the key 'W'
		If event.getEventType()=EET_KEY_INPUT_EVENT And event.getKeyPressedDown()=False
		
			Local key:Int=event.getKeyInputKey()

			Select key
				' switch wire frame mode
				Case EKEY_KEY_W  
					box.setMaterialFlag(EMF_WIREFRAME, box.getMaterial(0).getWireframe()=False)
					Return True	
			EndSelect			
		EndIf
		
		Return False
	EndMethod

	' we must override the generate function in order for instantiation to work properly
	' must return IEventReceiver
	Function generate:IEventReceiver()
		Return New MyEventReceiver
	EndFunction
EndType

window=CreateWindow("Irrlicht Win32 BlitzMax window example",100,100, ..
	windowWidth,windowHeight,Null,WINDOW_TITLEBAR | WINDOW_CLIENTCOORDS | WINDOW_HIDDEN)

' Create ok button

Local Ok:TGadget = ..
	CreateButton("OK - Close",windowWidth - 160,windowHeight - 40,150,30,window)

' create some text

CreateLabel("This is Irrlicht running inside a Win32 MaxGUI window." + Chr(13) ..
	+ "Also mixing with MFC and .NET Windows.Forms is possible.", 20, 20, 400, 40,window)

' Create window To put irrlicht in

Local hIrrlichtWindow:TGadget = ..
	CreateCanvas(50,80,320,220,window,PANEL_ACTIVE|PANEL_BORDER)

' this is not in the original, but is here to show you can have a textbox and enter into it
' without the events posting to Irrlicht
Local tb:TGadget = ..
	CreateTextField(windowWidth - 160,windowHeight - 70,150,20,window)

Rem
So now that we have some window, we can Create an Irrlicht device
inside of it. We use Irrlicht createEx() Function For this. We only
need the handle (HWND) To that window, set it as windowsID parameter
And start up the engine as usual. That's it.
EndRem
' Create irrlicht device in the button window

Local param:SIrrlichtCreationParameters=SIrrlichtCreationParameters.Create()
param.setWindowId(QueryGadget(hIrrlichtWindow,QUERY_HWND))
param.setDriverType(EDT_OPENGL)

' create the device from the params object
Local device:IrrlichtDevice= ..
	IrrlichtDevice.createFromParams(param)

' setup a simple 3d scene

Local smgr:ISceneManager=device.getSceneManager()
Local driver:IVideoDriver=device.getVideoDriver()

Local cam:ICameraSceneNode = smgr.addCameraSceneNode()
cam.setTarget(_VECTOR3DF(0,0,0))

Local anim:ISceneNodeAnimator = ..
	smgr.createFlyCircleAnimator(_VECTOR3DF(0,15,0),30.0)
cam.addAnimator(anim)
anim.drop()

Local cube:ISceneNode=smgr.addCubeSceneNode(20)

cube.setMaterialTexture(0, driver.getTexture("../media/wall.bmp"))
cube.setMaterialTexture(1, driver.getTexture("../media/water.jpg"))
cube.setMaterialFlag( EMF_LIGHTING, False )
cube.setMaterialType( EMT_REFLECTION_2_LAYER )

smgr.addSkyBoxSceneNode( ..
	driver.getTexture("../media/irrlicht2_up.jpg"), ..
	driver.getTexture("../media/irrlicht2_dn.jpg"), ..
	driver.getTexture("../media/irrlicht2_lf.jpg"), ..
	driver.getTexture("../media/irrlicht2_rt.jpg"), ..
	driver.getTexture("../media/irrlicht2_ft.jpg"), ..
	driver.getTexture("../media/irrlicht2_bk.jpg"))

' create the event receiver that toggles wireframe
Local receiver:IEventReceiver = ..
	IEventReceiver.Create(MyEventReceiver.generate)
MyEventReceiver(receiver).setBox(cube)
device.setEventReceiver(receiver)

' show And execute dialog

ShowGadget(window)

' do message queue

Rem
Now the only thing missing is the drawing loop using
IrrlichtDevice::run(). We do this as usual. But instead of this, there
is another possibility: You can also simply use your own message loop
using GetMessage, DispatchMessage And whatever. Calling
Device.run() will cause Irrlicht To dispatch messages internally too.
You need Not call Device.run() If you want To do your own message
dispatching loop, but Irrlicht will Not be able To fetch user Input
Then And you have To do it on your own using the window messages,
DirectInput, Or whatever.
EndRem

While(device.run())

	driver.beginScene(True, True)
	smgr.drawAll()
	driver.endScene()

	If PeekEvent() Then 
		PollEvent()
		Select EventID()
			Case EVENT_WINDOWCLOSE
				Exit
			Case EVENT_GADGETACTION
				If EventSource()=Ok
					PostEvent(CreateEvent(EVENT_WINDOWCLOSE,Ok))
				EndIf
			Default
				If EventID()=EVENT_KEYDOWN And EventData()=KEY_W
					Local event:SEvent=SEvent.Create()
					event.setEventType(EET_KEY_INPUT_EVENT)
					event.setKeyInputKey(EventData())
					device.postEventFromUser(event)				
				EndIf
		End Select
	End If		
	
Wend

' the alternative, own message dispatching loop without Device->run() would
' look like this:

Rem
While True
	If PeekEvent() Then 
		PollEvent()
		Select EventID()
		Case EVENT_WINDOWCLOSE
			Exit
		Case EVENT_GADGETACTION		
			If EventSource()=Ok
				PostEvent(CreateEvent(EVENT_WINDOWCLOSE,Ok))
			End If
		End Select
	End If		
	'// advance virtual time
	device.GetTimer().Tick()
	'// draw engine picture
	driver.beginScene(True, True, 0)
	smgr.drawAll()
	driver.endScene()
Wend
EndRem

device.closeDevice()
device.drop()

Rem
That's it, Irrlicht now runs in your own windows window.
EndRem
